/*******************************************************************************
 * Copyright (c) 2005, 2008 IBM Corporation and others. All rights reserved. This
 * program and the accompanying materials are made available under the terms of
 * the Eclipse Public License v1.0 which accompanies this distribution, and is
 * available at http://www.eclipse.org/legal/epl-v10.html $Id:
 * DeleteFileCommand.java,v 1.1 2005/05/03 14:32:54 sschneid Exp $
 * 
 * Contributors: IBM - Initial API and implementation
 ******************************************************************************/

package org.eclipse.hyades.internal.execution.core.file;

import java.io.File;
import java.io.IOException;
import java.util.Iterator;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.hyades.execution.core.file.IFileManagerExtended.Cookie;
import org.eclipse.hyades.execution.core.file.IFileManagerExtended.FileIdentifierList;
import org.eclipse.hyades.execution.core.internal.resources.CoreResourceBundle;
import org.eclipse.hyades.internal.execution.core.file.socket.ISocketChannel;
import org.eclipse.osgi.util.NLS;

/**
 * The delete file command, an abstract file manipulation command subclass used
 * for delete files on the server -- the delete file command supports bulk
 * operation removing a group of files in one operation
 * 
 * @author Scott E. Schneider
 */
class DeleteDirectoryCommand extends AbstractFileManipulationCommand implements IDeleteDirectoryCommand {

	/**
	 * The client side personality of the delete command
	 * 
	 * @author Scott E. Schneider
	 */
	private class Client extends AbstractFileManipulationCommand.Client {

		/**
		 * Creates the client side personality for the delete command
		 * 
		 * @param channel
		 *            the channel to communicate with the server
		 */
		Client(ISocketChannel channel) {
			super(channel);
		}

		/*
		 * (non-Javadoc)
		 * 
		 * @see org.eclipse.hyades.internal.execution.core.file.ICommand#execute()
		 */
		public void execute() throws IOException {

			// Initiate client
			super.execute();

			// Send the number of files to be removed
			FileIdentifierList remoteIdentifiers = DeleteDirectoryCommand.this.remoteIdentifiers;
			this.send(remoteIdentifiers.size());

			// Send all remote identifier names
			this.send(remoteIdentifiers.getArray());

		}
	}

	/**
	 * The server side personality of the delete command
	 * 
	 * @author Scott E. Schneider
	 */
	private class Server extends AbstractFileManipulationCommand.Server {

		/**
		 * Constructs the server side state personality for the delete command
		 * 
		 * @param channel
		 *            the channel to communicate with the client
		 */
		Server(ISocketChannel channel) {
			super(channel);
		}

		/*
		 * (non-Javadoc)
		 * 
		 * @see org.eclipse.hyades.internal.execution.core.file.ICommand#execute()
		 */
		public void execute() throws IOException {

			// Initiate server
			super.execute();

			// Receive number of files that will be deleted
			this.receiveInt();

			// Receive the remote identifiers from the client
			FileIdentifierList remoteIdentifiers = FileIdentifierList.create(this.receiveStrings());
			DeleteDirectoryCommand.this.remoteIdentifiers = remoteIdentifiers;

			// Remove all the files on the server side
			for (Iterator identifiers = remoteIdentifiers.iterator(); identifiers.hasNext();) {
				String fileName = (String) identifiers.next();
				File remoteDir = new File(fileName);
				if (remoteDir.exists() && !remoteDir.isFile()) {
					deleteDirectory(remoteDir);
					FileSystemServices.println(NLS.bind(CoreResourceBundle.DeleteDirectoryCommand_IDENTIFIED_REMOVED_, remoteDir.getAbsolutePath()), this);
				} else {
					FileSystemServices.println(NLS.bind(CoreResourceBundle.DeleteDirectoryCommand_NOT_IDENTIFIED_REMOVED_, remoteDir.getAbsolutePath()), this);
				}
			}

		}
		
		private void deleteDirectory(File directory) {
			File[] files = directory.listFiles();
			for (int i = 0; i < files.length; i++) {
				if (files[i].isFile()) {
					files[i].delete();
				} else {
					deleteDirectory(files[i]);
				}
			}
		}
	}

	/**
	 * Constructs a delete file command given the specified cookie and monitor
	 * instances
	 * 
	 * @param cookie
	 *            the unique identity identifying the cached server side
	 *            operands of this operation
	 * @param monitor
	 *            the progress monitor used for progress and cancel support
	 */
	DeleteDirectoryCommand(Cookie cookie, IProgressMonitor monitor) {
		super(DeleteDirectoryCommand.class, cookie, monitor);
	}

	/**
	 * Constructs a delete file command given the specified channel
	 * 
	 * @param channel
	 *            the channel to communicate with the client on
	 */
	public DeleteDirectoryCommand(ISocketChannel channel) {
		super(DeleteDirectoryCommand.class);
		this.setState(new Server(channel));
	}

	/**
	 * Constructs a delete file command given the specified channel, remote
	 * identifier operands and progress monitor
	 * 
	 * @param channel
	 *            the channel to communicate with the server on
	 * @param remoteIdentifiers
	 *            the remote identifier operands
	 * @param monitor
	 *            the progress monitor used for progress nad cancel control
	 */
	DeleteDirectoryCommand(ISocketChannel channel, FileIdentifierList remoteIdentifiers, IProgressMonitor monitor) {
		super(DeleteDirectoryCommand.class, remoteIdentifiers, monitor);
		this.setState(new Client(channel));

	}

}
