/**********************************************************************
 * Copyright (c) 2005, 2011 IBM Corporation and others.
 * All rights reserved.   This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * $Id: KeystoreHelper.java,v 1.6 2008-03-20 18:49:49 dmorris Exp $
 * 
 * Contributors: 
 * IBM - Initial API and implementation
 **********************************************************************/

package org.eclipse.hyades.execution.security;

/**
 * @author rduggan
 *
 */
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.security.KeyStore;
import java.security.KeyStoreException;
import java.security.NoSuchAlgorithmException;
import java.security.NoSuchProviderException;
import java.security.cert.Certificate;
import java.security.cert.CertificateException;
import java.security.cert.CertificateFactory;
import java.util.HashMap;
import java.util.Map;

import org.eclipse.hyades.internal.execution.local.control.Node;

public class KeystoreHelper {
	
	/* Map from certificate to connected Node with connections under that certificate */
	private static Map<Certificate, Node> activeCertificates = new HashMap<Certificate, Node>();
	
	public static KeyStore createKeyStore(String pathname, String password)
	   throws KeyStoreException,
	           NoSuchAlgorithmException,
	           CertificateException,
	           IOException,
	           NoSuchProviderException  {
	           	
	    KeyStore ks=KeyStore.getInstance("JKS");//$NON-NLS-1$
		
		File file=new File(pathname);
		
		/* Do not stomp an existing file */
		/* bug 197177, handle situation when dat file is of size 0. */
		if(!file.exists() || file.length() == 0) {
			ks.load(null, password.toCharArray());
			persistKeyStore(ks, pathname, password);
		}
		else {
			return loadKeyStore(pathname, password);
		}
		return ks;
	}
	
	public static KeyStore loadKeyStore(String pathname, String password)
	   throws KeyStoreException,
	           NoSuchAlgorithmException,
	           CertificateException,
	           IOException,
	           NoSuchProviderException  {
		
		KeyStore ks=null;
		File file=new File(pathname);
		
		/* Do not stomp an existing file. */
		/* bug 197177.  Add check for file size in case this method is called by others */
		if(file.exists()&& file.length() > 0) {
			ks=KeyStore.getInstance("JKS");//$NON-NLS-1$
			/* Initialize the keystore with no information */
			FileInputStream is=new FileInputStream(file);
			ks.load(is, password.toCharArray());
			is.close();
		}
		return ks;
	}
	
	public static Certificate loadCertificate(String certFilename) 
	   throws CertificateException,
	           FileNotFoundException {
		
		CertificateFactory factory=CertificateFactory.getInstance("X.509");//$NON-NLS-1$
		
		return factory.generateCertificate(new FileInputStream(certFilename));
	}
	
	public static void addCertificateToKeyStore(KeyStore ks, Certificate cert, String alias)
		      throws KeyStoreException {
		ks.setCertificateEntry(alias, cert);
			
	}
	
	public static void persistKeyStore(KeyStore ks, String pathname, String password) 
		throws KeyStoreException,
		        FileNotFoundException,
		        NoSuchAlgorithmException,
		        CertificateException,
		        IOException {
		FileOutputStream os=new FileOutputStream(pathname);
		ks.store(os, password.toCharArray());
		os.close();
		
		
	}
	
	/**
	 * Records an association between the `cert' and `node', indicating that the connection
	 * to `node' has been made under the trust of certificate `cert'.
	 * 
	 * This book-keeping is maintained so that the certificate UI can provide intelligent
	 * handling of the case where a user revokes the trust of a certificate that has an
	 * active connection.
	 * 
	 * @param cert
	 * @param node
	 */
	public static void associateCertificateWithNode( Certificate cert, Node node ) {
	
		synchronized (activeCertificates) {
			activeCertificates.put( cert, node );			
		}
		
	}

	/**
	 * Dissociate the link between `cert' and `node'. This should be called when the `node's 
	 * connection has been terminated.
	 * 
	 * @param cert
	 * @param node
	 */
	public static void dissociateCertificateFromNode( Certificate cert, Node node ) {
		
		synchronized (activeCertificates) {
			if( node == activeCertificates.get(cert) ) {
				activeCertificates.remove(cert);
			}			
		}
		
	}

	/**
	 * Return the Node which has an active connection under `cert'. If there is no active connection
	 * then this method returns null.
	 * 
	 * @param cert
	 * @return The node associated with the activate connection under `cert', or null if there isn't one
	 */
	public static Node lookupCertificateAssociation( Certificate cert ) {
		synchronized (activeCertificates) {
			return activeCertificates.get(cert);
		}
	}
	
}
